@@ -6,6 +6,15 @@
   Contents:     Electronic logbook utility   
 
   $Log: elog.c,v $
+  Revision 1.22  2004/11/02 16:46:55  midas
+  Added -H flag for HTML submit
+
+  Revision 1.21  2004/09/18 04:42:46  midas
+  Fixed bug with not displaying inline images
+
+  Revision 1.20  2004/03/28 10:41:05  midas
+  Fixed problem with logbooks containing blanks
+
   Revision 1.19  2004/03/26 08:28:29  midas
   Version 2.5.2
 
@@ -260,6 +269,33 @@ void btou(char *str)
 
 /*------------------------------------------------------------------*/
 
+void url_encode(char *ps, int size)
+/********************************************************************\
+Encode the given string in-place by adding %XX escapes
+\********************************************************************/
+{
+   unsigned char *pd, *p, str[NAME_LENGTH];
+
+   pd = str;
+   p = ps;
+   while (*p && (int) pd < (int) str + 250) {
+      if (strchr("%&=#?+", *p) || *p > 127) {
+         sprintf(pd, "%%%02X", *p);
+         pd += 3;
+         p++;
+      } else if (*p == ' ') {
+         *pd++ = '+';
+         p++;
+      } else {
+         *pd++ = *p++;
+      }
+   }
+   *pd = '\0';
+   strlcpy(ps, str, size);
+}
+
+/*------------------------------------------------------------------*/
+
 void sgets(char *string, int size)
 {
    char *p;
@@ -278,8 +314,7 @@ char request[100000], response[100000], *content;
 
 INT retrieve_elog(char *host, int port, char *subdir, char *experiment,
                   char *passwd, char *uname, char *upwd, int message_id,
-                  char attrib_name[MAX_N_ATTR][NAME_LENGTH],
-                  char attrib[MAX_N_ATTR][NAME_LENGTH], char *text)
+                  char attrib_name[MAX_N_ATTR][NAME_LENGTH], char attrib[MAX_N_ATTR][NAME_LENGTH], char *text)
 /********************************************************************\
 
   Routine: retrive_elog
@@ -528,6 +563,7 @@ INT submit_elog(char *host, int port, char *subdir, char *experiment,
                 int reply,
                 int edit,
                 int suppress,
+                int html,
                 char attrib_name[MAX_N_ATTR][NAME_LENGTH],
                 char attrib[MAX_N_ATTR][NAME_LENGTH],
                 int n_attr,
@@ -549,6 +585,7 @@ INT submit_elog(char *host, int port, char *subdir, char *experiment,
     int    reply            Reply to existing message
     int    edit             Edit existing message
     int    suppress         Suppress Email notification
+    int    html             Submit as HTML
     char   *attrib_name     Attribute names
     char   *attrib          Attribute values
     char   *text            Message text
@@ -644,7 +681,6 @@ INT submit_elog(char *host, int port, char *subdir, char *experiment,
       if (i < n_attr)
          encoding = attrib[i];
 
-
       strlcpy(new_text, text, sizeof(new_text));
 
       /* precede old text with "> " */
@@ -752,42 +788,39 @@ INT submit_elog(char *host, int port, char *subdir, char *experiment,
 
    if (uname[0])
       sprintf(content + strlen(content),
-              "%s\r\nContent-Disposition: form-data; name=\"unm\"\r\n\r\n%s\r\n",
-              boundary, uname);
+              "%s\r\nContent-Disposition: form-data; name=\"unm\"\r\n\r\n%s\r\n", boundary, uname);
 
    if (upwd[0]) {
       base64_encode(upwd, str);
       sprintf(content + strlen(content),
-              "%s\r\nContent-Disposition: form-data; name=\"upwd\"\r\n\r\n%s\r\n",
-              boundary, str);
+              "%s\r\nContent-Disposition: form-data; name=\"upwd\"\r\n\r\n%s\r\n", boundary, str);
    }
 
    if (experiment[0])
       sprintf(content + strlen(content),
-              "%s\r\nContent-Disposition: form-data; name=\"exp\"\r\n\r\n%s\r\n",
-              boundary, experiment);
+              "%s\r\nContent-Disposition: form-data; name=\"exp\"\r\n\r\n%s\r\n", boundary, experiment);
 
    if (reply)
       sprintf(content + strlen(content),
-              "%s\r\nContent-Disposition: form-data; name=\"reply_to\"\r\n\r\n%d\r\n",
-              boundary, reply);
+              "%s\r\nContent-Disposition: form-data; name=\"reply_to\"\r\n\r\n%d\r\n", boundary, reply);
 
    if (edit)
       sprintf(content + strlen(content),
-              "%s\r\nContent-Disposition: form-data; name=\"edit_id\"\r\n\r\n%d\r\n",
-              boundary, edit);
+              "%s\r\nContent-Disposition: form-data; name=\"edit_id\"\r\n\r\n%d\r\n", boundary, edit);
 
    if (suppress)
       sprintf(content + strlen(content),
-              "%s\r\nContent-Disposition: form-data; name=\"suppress\"\r\n\r\n1\r\n",
-              boundary);
+              "%s\r\nContent-Disposition: form-data; name=\"suppress\"\r\n\r\n1\r\n", boundary);
+
+   if (html)
+      sprintf(content + strlen(content),
+              "%s\r\nContent-Disposition: form-data; name=\"html\"\r\n\r\n1\r\n", boundary);
 
    for (i = 0; i < n_attr; i++) {
       strcpy(str, attrib_name[i]);
       btou(str);
       sprintf(content + strlen(content),
-              "%s\r\nContent-Disposition: form-data; name=\"%s\"\r\n\r\n%s\r\n", boundary,
-              str, attrib[i]);
+              "%s\r\nContent-Disposition: form-data; name=\"%s\"\r\n\r\n%s\r\n", boundary, str, attrib[i]);
    }
 
    sprintf(content + strlen(content),
@@ -818,12 +851,14 @@ INT submit_elog(char *host, int port, char *subdir, char *experiment,
    strcpy(request, "POST /");
    if (subdir[0])
       sprintf(request + strlen(request), "%s/", subdir);
-   if (experiment[0])
-      sprintf(request + strlen(request), "%s/", experiment);
+   if (experiment[0]) {
+      strcpy(str, experiment);
+      url_encode(str, sizeof(str));
+      sprintf(request + strlen(request), "%s/", str);
+   }
    strcat(request, " HTTP/1.0\r\n");
 
-   sprintf(request + strlen(request),
-           "Content-Type: multipart/form-data; boundary=%s\r\n", boundary);
+   sprintf(request + strlen(request), "Content-Type: multipart/form-data; boundary=%s\r\n", boundary);
    sprintf(request + strlen(request), "Host: %s\r\n", host_name);
    sprintf(request + strlen(request), "User-Agent: ELOG\r\n");
    sprintf(request + strlen(request), "Content-Length: %d\r\n", content_length);
@@ -929,12 +964,12 @@ int main(int argc, char *argv[])
    char host_name[256], logbook[32], textfile[256], password[80], subdir[256];
    char *buffer[MAX_ATTACHMENTS], attachment[MAX_ATTACHMENTS][256];
    INT att_size[MAX_ATTACHMENTS];
-   INT i, n, fh, n_att, n_attr, size, port, reply, edit, suppress;
+   INT i, n, fh, n_att, n_attr, size, port, reply, edit, suppress, html;
    char attr_name[MAX_N_ATTR][NAME_LENGTH], attrib[MAX_N_ATTR][NAME_LENGTH];
 
    text[0] = textfile[0] = uname[0] = upwd[0] = suppress = 0;
    host_name[0] = logbook[0] = password[0] = subdir[0] = 0;
-   n_att = n_attr = reply = edit = 0;
+   n_att = n_attr = reply = edit = html = 0;
    port = 80;
 
    for (i = 0; i < MAX_ATTACHMENTS; i++) {
@@ -949,6 +984,8 @@ int main(int argc, char *argv[])
          verbose = 1;
       else if (argv[i][0] == '-' && argv[i][1] == 'x')
          suppress = 1;
+      else if (argv[i][0] == '-' && argv[i][1] == 'H')
+         html = 1;
       else {
          if (argv[i][0] == '-') {
             if (i + 1 >= argc || argv[i + 1][0] == '-')
@@ -974,8 +1011,7 @@ int main(int argc, char *argv[])
                   strcpy(attr_name[n_attr], str);
                   n_attr++;
                } else {
-                  printf
-                      ("Error: Attributes must be supplied in the form \"-a <attribute>=<value>\".\n");
+                  printf("Error: Attributes must be supplied in the form \"-a <attribute>=<value>\".\n");
                   return 1;
                }
             } else if (argv[i][1] == 'f')
@@ -990,28 +1026,21 @@ int main(int argc, char *argv[])
              usage:
                printf("\nusage: elog\n");
                printf("           -h <hostname> [-p port] [-s subdir]\n");
-               printf
-                   ("                                    Location where elogd is running\n");
-               printf
-                   ("           -l logbook/experiment    Name of logbook or experiment\n");
+               printf("                                    Location where elogd is running\n");
+               printf("           -l logbook/experiment    Name of logbook or experiment\n");
                printf("           [-v]                     for verbose output\n");
-               printf
-                   ("           [-w password]            write password defined on server\n");
+               printf("           [-w password]            write password defined on server\n");
                printf("           [-u username password]   user name and password\n");
-               printf("           [-f <attachment>]        (up to %d times)\n",
-                      MAX_ATTACHMENTS);
-               printf("           -a <attribute>=<value>   (up to %d times)\n",
-                      MAX_N_ATTR);
+               printf("           [-f <attachment>]        (up to %d times)\n", MAX_ATTACHMENTS);
+               printf("           -a <attribute>=<value>   (up to %d times)\n", MAX_N_ATTR);
                printf("           [-r <id>]                Reply to existing message\n");
                printf("           [-e <id>]                Edit existing message\n");
-               printf
-                   ("           [-x]                     Suppress email notification\n");
+               printf("           [-x]                     Suppress email notification\n");
+               printf("           [-H]                     Submit entry in HTML format\n");
                printf("           -m <textfile>] | <text>\n");
                printf("\nArguments with blanks must be enclosed in quotes\n");
-               printf
-                   ("The elog message can either be submitted on the command line, piped in like\n");
-               printf
-                   ("\"cat text | elog -h ... -l ... -a ...\" or in a file with the -m flag.\n");
+               printf("The elog message can either be submitted on the command line, piped in like\n");
+               printf("\"cat text | elog -h ... -l ... -a ...\" or in a file with the -m flag.\n");
                printf("Multiple attributes and attachments can be supplied\n");
                return 1;
             }
@@ -1048,8 +1077,7 @@ int main(int argc, char *argv[])
       lseek(fh, 0, SEEK_SET);
 
       if (size > (int) sizeof(text) - 1) {
-         printf("Message file \"%s\" is too long (%d bytes max).\n", textfile,
-                sizeof(text));
+         printf("Message file \"%s\" is too long (%d bytes max).\n", textfile, sizeof(text));
          return 1;
       }
 
@@ -1108,7 +1136,7 @@ int main(int argc, char *argv[])
 
    /* now submit message */
    submit_elog(host_name, port, subdir, logbook, password,
-               uname, upwd, reply, edit, suppress, attr_name, attrib, n_attr, text,
+               uname, upwd, reply, edit, suppress, html, attr_name, attrib, n_attr, text,
                attachment, buffer, att_size);
 
    for (i = 0; i < MAX_ATTACHMENTS; i++)
